local DELAY = 2
local PRISONER_PATIENCE = 5
local EMEMPT_COOLDOWN = 3
local Now   = Game.Time
local Log   = Game.DebugOut
local ready = Now() + DELAY
local ExemptPrisoners = {}
local ExemptPrisonersTest = {}
local PatdownStations = {}

function Update( elapsedTime )
	-- Search, clear pathfinding, & exempt prisoner.
	PatdownStationSearch()
	
	-- Delayed update of Patdown Stations (every 2s)
	if (Now() > ready) then
		-- Fetch Patdown Stations on the map, and store it in PatdownStations
		PatdownStations = FetchObjectsGlobal("PatdownStation")
		
		-- Update ready.
		ready = Now() + DELAY
	end
	
	if (table.getn(ExemptPrisoners) > 1) then
		DoPatdownExemptCheck()
	end
end

-- Function that empties tables passsed into it.
function EmptyTable(tableToEmpty)
	for k in pairs (tableToEmpty) do
		tableToEmpty [k] = nil
	end
end

-- Function returns all objects of a type
function FetchObjectsGlobal(objectName)
	-- Create dummy object
	local dummyObj = Object.Spawn('Box', -1, -1);
	
	-- Fetch Object using dummy object
	local objects = dummyObj.GetNearbyObjects(objectName, 10000);
	
	-- Delete dummy object
	Object.Delete(dummyObj);
	
	-- Return fetched objects
	return objects
end

function FindPrisonersNearPatdownStations()
	-- Fetch patdown stations on the map
	patdownStations = FetchObjectsGlobal("PatdownStation")
	-- Fetch nearby prisoners
	local nearbyPrisoners = patdownStation.GetNearbyObjects('Prisoner', 1);
	-- Return nearby Prisoners
	return nearbyPrisoners
end

-- Manages prisoner timings.
function DoPatdownExemptCheck()
	-- Loop for each prisoner within allPrisoners
	for a in pairs (ExemptPrisonersTest) do
		prisoner = ExemptPrisonersTest[a]
		-- Loop for each table within ExemptPrisoners
		for k in pairs (ExemptPrisoners) do
			-- Fetch the nestedTable
			local nestedTable = ExemptPrisoners [k]
			
			-- Check if prisoner.Id.i == nestedTable[1] 
			if prisoner.Id.i == nestedTable[1] then
			
				-- Check if PRISONER_PATIENCE is up
				if     Now() < nestedTable[2] + PRISONER_PATIENCE then
					-- Clear prisoner routing
					prisoner.ClearRouting()
				
				-- Check if EMEMPT_COOLDOWN is up
				elseif Now() > nestedTable[2] + PRISONER_PATIENCE + EMEMPT_COOLDOWN then
					-- Remove them from ExemptPrisoners
					table.remove(ExemptPrisoners,k)
					table.remove(ExemptPrisonersTest, a)
				end
			end
		end
	end
end

-- Search, clear pathfinding, & exempt prisoner.
function PatdownStationSearch()
	-- Loop for each patdown station
	for patdownStation, rng in pairs(PatdownStations) do
	
		-- Fetch nearby prisoners
		local prisonersList = patdownStation.GetNearbyObjects('Prisoner', 1);
	
		-- Loop for each prisoner near Patdown Stations.
		for prisoner, rng in pairs(prisonersList) do
		
			isExemptPrisoner = false
			
			-- Loop for each table within ExemptPrisoners
			for k in pairs (ExemptPrisoners) do
			
				-- Fetch the nestedTable
				local nestedTable = ExemptPrisoners [k]
				
				-- Check if prisoner.Id.i == nestedTable[1] 
				if prisoner.Id.i == nestedTable[1] then
					-- Exempt this prisoner.
					isExemptPrisoner = true
				end
			end
			
			if isExemptPrisoner == false then
				-- Clear prisoner routing
				prisoner.ClearRouting()
				-- Request search of prisoner
				WorkQueue.Request('SearchObject', prisoner)
				-- Add prisoner to ExemptPrisoners
				table.insert(ExemptPrisoners, {prisoner.Id.i, Now()})
				table.insert(ExemptPrisonersTest, prisoner)
			end
		end
	end
end